#!/usr/bin/python
#
# glcu - gentoo linux cron update
#
# written by Michael Panhorst Michael [at] Panhorst [dot] com
#
# released under the GPL-2
#
# Website: http://www.panhorst.com/glcu/


import sys
import os
import ConfigParser
import getopt
import output
import sre
import pickle
import string
import sets
import datetime

VERSION = "0.9.3"


fullLog = ''

def exception (errMsg):
    print "!!! ERROR: " + errMsg
    sys.exit(9)

def printHelp():
    print "  glcu - "+output.bold("g")+"entoo "+output.bold("l")+"inux "+output.bold("c")+"ron "+output.bold("u")+"pdate"
    print "       a program to keep your gentoo linux up to date!"
    print "       see: http://www.panhorst.com/glcu/ for more information\n"
    print "                 ( Version" , VERSION , ")\n"
    print "USAGE:\n"
    print "  glcu [-u|-v|-A|-S|-h|-e(E)|-f(F)|-t(T)] [update-file]\n"
    print "      -u: update 'security', 'system' or 'world'"
    print "      -v: verbosity level (0:quiet, 1:normal, 2:verbose)"
    print "      -a: run glcu in automatic mode (don't ask anything) | or not: -A"
    print "      -S: don't run emerge sync"
    print "      -h: show this help message"
    print "      -e: run eupdatedb | or not: -E"
    print "      -f: run fixpackages if necessary | or not: -F"
    print "      -r: remove prebuilt binary packages after installation | or not: -R"
    print
    sys.exit(0)


    
class GlcuConfig:
    """
    Config Class for glcu
        1. initialize standard values
        2. read configuration file
        3. check command line options if run manually    
    """
    
    __mainConfig = {'email':'root',
                    'verbosity':'1',
                    'update':'system',
                    'tmpdir':'/tmp',
                    'sysworldoptions':'--deep --update',
                    'cronday':'7',
                    'bannedpackages':('gentoo-dev-sources','vanilla-sources','nvidia-kernel'),
                    'sync':True,
                    'eupdatedb':True,
                    'fixpackages':True,
                    'security':True,
                    'removeprebuilt':True,
                    'automatic':False}
    
    def __init__(self,name):
        self.name = name
        if (self.name == 'cron') or (self.name == 'manual'):
            self.__readConfigFile()
        if (self.name == 'manual'):
            self.__readCommandLineOptions()
                
                
    def getMainConfig(self,getName):
        return self.__mainConfig[getName]

    def setMainConfig(self,setName,setValue):
        __setNew = {setName:setValue}
        self.__mainConfig.update(__setNew)

    def __readConfigFile(self):
        configFile = ConfigParser.ConfigParser()
        configFile.readfp(open('/etc/conf.d/glcu'))
        pairs = configFile.items('default')
        for pair in pairs:
            self.setMainConfig(pair[0],pair[1])

        boolkeys = configFile.options('bool')
        for boolkey in boolkeys:
            booloption = configFile.getboolean('bool',boolkey)
            self.setMainConfig(boolkey,booloption)
            
        if (int(self.getMainConfig('verbosity')) > 2): # debugging:  only in /etc/conf.d/glcu for verbosity = 3 
            print "\nFrom __readConfigFile:"
            for key in self.__mainConfig.keys():
                print "  " + key + ":" , self.__mainConfig[key] 
            print
    
            
    def __readCommandLineOptions(self):
        commandLineOptions,arguments = getopt.gnu_getopt(sys.argv[1:],'aAd:eEfFhrRSu:v:?', ['automatic','noautomatic','tmpdir=','eupdatedb','noeupdatedb','fixpackages','nofixpackages','help','removeprebuilt','noremoveprebuilt','nosync','update=','verbosity='])
       
        # first check for collisions of the command line options:
        flags = []
        for checkflag in commandLineOptions:
            flags.append(checkflag[0])
        
        if (('-F' or '--nofixpackages') in flags) and (('-f' or '--fixpackages') in flags):
            exception('Collision of commandline options: -f/-F')
        elif (('-E' or '--noeupdatedb') in flags) and (('-e' or '--eupdatedb') in flags):
            exception('Collision of commandline options: -e/-E')
        elif (('-R' or '--noremoveprebuilt') in flags) and (('-r' or '--removeprebuilt') in flags):
            exception('Collision of commandline options: -r/-R')

        # then set the command line options:
        for clo in commandLineOptions:

            if (clo[0] == '-h') or (clo[0] == '--help') or (clo[0] == '-?'):
                printHelp()
                
            elif (clo[0] == '-A') or (clo[0] == '--noautomatic'):
                self.setMainConfig('automatic',False)
                sys.argv.remove(clo[0])
                
            elif (clo[0] == '-a') or (clo[0] == '--automatic'):
                self.setMainConfig('automatic',True)
                sys.argv.remove(clo[0])
                
            elif (clo[0] == '-S') or (clo[0] == '--nosync'):
                self.setMainConfig('sync',False)
                sys.argv.remove(clo[0])
                
            elif (clo[0] == '-F') or (clo[0] == '--nofixpackages'):
                self.setMainConfig('fixpackages',False)
                sys.argv.remove(clo[0])
                
            elif (clo[0] == '-f') or (clo[0] == '--fixpackages'):
                self.setMainConfig('fixpackages',True)
                sys.argv.remove(clo[0])
            
            elif (clo[0] == '-E') or (clo[0] == '--noeupdatedb'):
                self.setMainConfig('eupdatedb',False)
                sys.argv.remove(clo[0])
                
            elif (clo[0] == '-e') or (clo[0] == '--eupdatedb'):
                self.setMainConfig('eupdatedb',True)   
                sys.argv.remove(clo[0])
            
            elif (clo[0] == '-R') or (clo[0] == '--noremoveprebuilt'):
                self.setMainConfig('removeprebuilt',False)
                sys.argv.remove(clo[0])
                
            elif (clo[0] == '-r') or (clo[0] == '--removeprebuilt'):
                self.setMainConfig('removeprebuilt',True)
                sys.argv.remove(clo[0])
            
            elif (clo[0] == '-u') or (clo[0] == '--update'):
                if (clo[1] == 'security') or (clo[1] == 'system') or (clo[1] == 'world'):
                    self.setMainConfig('update',clo[1])
                    sys.argv.remove(clo[0])
                    sys.argv.remove(clo[1])
                else:
                    exception('wrong argument: --update ' + clo[1])
                
            elif (clo[0] == '-v') or (clo[0] == '--verbosity'):
                if (int(clo[1]) >= 0):
                    self.setMainConfig('verbosity',int(clo[1]))
                    sys.argv.remove(clo[0])
                    sys.argv.remove(clo[1])
                else:
                    exception('wrong verbosity argument: ' + clo[1])
            elif (clo[0] == '-d') or (clo[0] == '--tmpdir'):
                if (os.path.isdir(clo[1]) == 1):
                    self.setMainConfig('tmpdir',clo[1])
                    sys.argv.remove(clo[0])
                    sys.argv.remove(clo[1])
                else:
                    exception(clo[1] + ' is not a directory!')
            

        #debugging only
        if (int(self.getMainConfig('verbosity')) > 2):
            print "\nFrom __readCommandLineOptions:"
            for key in self.__mainConfig.keys():
                print "  " + key + ":" , self.__mainConfig[key] 
            print



class ShellExecution:

    def __init__ (self,inputCommand):
        __outName = mainConfig.getMainConfig('tmpdir') + '/glcu-out.' + str(os.getpid())
        self.shellCommand = inputCommand + ' >' + __outName + ' 2>&1'
        if (int(mainConfig.getMainConfig('verbosity')) > 2): #debugging
            print "shellExecution:" , ":" , self.shellCommand
        
        self.__exitStatus = os.system(self.shellCommand)/256    
        
        __o = file(__outName,'r')
        self.__output = __o.read()
        __o.close()
        os.remove(__outName)
        
        
        if (int(mainConfig.getMainConfig('verbosity')) > 2): #debugging
            print "Output:" , self.__output
            print "exit status:" , self.__exitStatus
            
 

        global fullLog
        if ((int(mainConfig.getMainConfig('verbosity')) > 1) and int(self.__exitStatus) == 0):
            fullLog = fullLog + 'Log for Command: ' + self.shellCommand + "\n" + 70*"-" + "\n"
            fullLog = fullLog + self.__output
            fullLog = fullLog + 70*"=" + "\n"

    def getOutput(self):
        return self.__output
    
    def getExitStatus(self):
        return self.__exitStatus

class CronOutput:

    __packageSuccess = []
    __packageError = []
    __packageDependency = []
    __packageExists = []
    __packageBlocks = []
    __fileName = '/tmp' + '/glcuUpdate-' + str(os.getpid())
    __errorLog = ''
    
    def __init__ (self):
        pass
                
    # divide packages in working and error packages
    def addPackage(self,package,status,packageOutput):
        self.package = package
        self.status = status
        
        if (self.status == 0):
            self.__packageSuccess.append(self.package)
        elif (self.status < 0):
            self.__packageDependency.append(self.package)    
        else:
            self.__packageError.append(self.package)
            self.__addErrorAppendix(self.package,packageOutput)
            
        if (int(mainConfig.getMainConfig('verbosity')) > 2): # debugging only    
            print "AddErrorPackages:" , self.__packageError
            print "AddDependencyPackages:" , self.__packageDependency
            print "AddSuccessPackages:" , self.__packageSuccess

            
    def addExistingPackage(self,package):
        self.package = package
        self.__packageExists.append(self.package)
        

    def addBlockPackage(self,package):
        self.package = package
        self.__packageBlocks.append(self.package)
        
        
    # get full output of the log-file because there was an error during prebuilding:
    def __addErrorAppendix(self,package,output):
        self.package = package
        self.output = output
        
        self.__errorLog = self.__errorLog + "!!! Error log for " + self.package + ":\n\n" + self.output+ 70*"-" + "\n"
        
        pass
    
    
    
    def __writeTmp(self):
        # write __packageSuccess (and existing) Packages with pickle to a tmpfile 
        # if there are packages in Dep or Error add update:system/world at the beginning of the list
        
        dumpPackages = self.__packageSuccess[:]
        dumpPackages.extend(self.__packageExists[:])

        if (len(self.__packageDependency) > 0) or (len(self.__packageError) > 0):
            dumpPackages.append(mainConfig.getMainConfig('update'))
        
        tempFile = file(self.__fileName,'w')
        pickle.dump(dumpPackages,tempFile)
        tempFile.close()
        os.chmod(self.__fileName,0600)
    
    def writeMail(self):
    
        self.__writeTmp()
        # get error logs (for __packageError) and appendix if wanted
        # build eMail text and mail it
        global hostname
        hostname = ''
        if (os.getenv('HOSTNAME')):
            hostname = '(' + str(os.getenv('HOSTNAME')) + ')'

        subject = 'glcu' + hostname + ': Updates for your Gentoo box available!'

        message = "  New Packages for 'emerge " + mainConfig.getMainConfig('update')\
        + "' available!\n  Update your Gentoo box with:\n\n          glcu "\
        + self.__fileName 
        if (len(self.__packageBlocks) > 0):
            message = message + "\n\n  You must resolve all blocks before glcu can update your system!"
        
        if (int(mainConfig.getMainConfig('verbosity')) > 0):
        
            message = message + "\n\n\nPrebuilding new Packages:\n" + 70*"-" + "\n"
            
            for package in self.__packageBlocks:
                message = message + string.ljust(package,57) + "BLOCK!\n"
            
            for package in self.__packageSuccess:
                message = message + string.ljust(package,57) + "success\n"
    
            for package in self.__packageExists:
                message = message + string.ljust(package,57) + "existing\n"
                
            for package in self.__packageDependency:
                message = message + string.ljust(package,57) + "dependencies*\n"
    
            for package in self.__packageError:
                message = message + string.ljust(package,57) + "ERROR**\n"
    
            message = message + 70*"-" + "\n"
            if (len(self.__packageDependency) > 0):
                message = message + "  *  Package depends on other new packages and can't be prebuild\n"
                
            if (len(self.__packageError) > 0):
                message = message + "  ** Error during prebuilding. See below for more information\n\n"
    
            if (len(self.__errorLog) > 0):
                message = message + "\n\n\nError logs:\n" +70*"=" + "\n" + self.__errorLog 
        
        
        if (int(mainConfig.getMainConfig('verbosity')) > 1):
            message = message + "\n\n" +70*"=" + "\n\n\nFull logs:\n" +70*"=" + "\n" + fullLog
    
        self.__eMail(subject,message)
        
    
    def earlyErrorMail(self,errorsubject,message):
        # if an early error happens (e.g. during emerge sync or eupdatedb)
        hostname = ''
        if (os.getenv('HOSTNAME')):
            hostname = '(' + str(os.getenv('HOSTNAME')) + ')'
            
        subject = 'glcu' + hostname + ' ERROR: ' + errorsubject
        self.__eMail(subject,message)
        sys.exit(23)
    
    def __eMail(self,subject,body):
        # The core eMail function
        mailFile = mainConfig.getMainConfig('tmpdir') + '/glcuMail' + str(os.getpid()) 
        mailBody = file(mailFile,'w')
        mailBody.write(body)
        mailBody.close()
        mailcommand = '/bin/cat ' + mailFile + '|/bin/mail -s "' + subject + '" ' + mainConfig.getMainConfig('email')
        mailObject = ShellExecution(mailcommand)
        if (mailObject.getExitStatus() > 0):
            mailBody = file(mailFile,'a')
            mailBody.write('ERROR while sending eMail:\n--------------------------\n\n' + mailObject.getOutput())
            mailBody.close()
            sys.exit(100)
        else:
            os.remove(mailFile)
            
        if (int(mainConfig.getMainConfig('verbosity')) > 2): # debugging only
            print "--> sending mail to: " + mainConfig.getMainConfig('email')
        


        
def cronExecution ():
    # load configuration:
    global mainConfig
    mainConfig = GlcuConfig('cron')
    if (int(mainConfig.getMainConfig('verbosity')) > 2): # debugging only
        print "cronExecution: " , sys.argv[0]

    #only run on the specified day
    myToday = datetime.date.today()
    myWeekDay = '%d' % myToday.isoweekday()    
    if ((int(mainConfig.getMainConfig('cronday')) != int(myWeekDay)) and (int(mainConfig.getMainConfig('cronday')) != 0)):
        sys.exit(0)
    
    #initialize mail object:
    mail = CronOutput()
    
    
    # Check if glcu file was upated. Otherwise sendmail and write check file
    if (int(mainConfig.getMainConfig('cronday')) == 8):
        if os.path.isfile('/tmp/glcuConfigNotUpdated'):
            sys.exit(1)
        else:
            mail.earlyErrorMail('Update config file!',"Important message from glcu:\n============================\n\n  Edit the glcu config file '/etc/conf.d/glcu' to suit your needs!\n  Otherwise glcu won't work.")
            nO = file('/tmp/glcuConfigNotUpdated','w')
            nO.write("edit '/etc/glcu' and delete this file!")
            nO.close()
            
    # 1. emerge sync

    if (mainConfig.getMainConfig('sync')):
        sync = ShellExecution('emerge sync')
        if (sync.getExitStatus() != 0):
            mail.earlyErrorMail('emerge sync failed','Error log for emerge sync:\n\n' + sync.getOutput())

        
        # 2. fixpackages (if wanted and needed)
        
        if (mainConfig.getMainConfig('fixpackages')):
            if (sre.search("Skipping packages. Run 'fixpackages' or set it in FEATURES",sync.getOutput())):
                fixpackages = ShellExecution('/usr/sbin/fixpackages')
                if (fixpackages.getExitStatus() != 0):
                    mail.earlyErrorMail('fixpackages failed','Error log for fixpackages:\n\n' + fixpackages.getOutput())
    

    # 3. run eupdatedb 

    if (mainConfig.getMainConfig('eupdatedb')):
        eupdatedb = ShellExecution('/usr/sbin/eupdatedb')
        if (eupdatedb.getExitStatus() != 0):
            mail.earlyErrorMail('eupdatedb failed','Error log for eupdatedb:\n\n' + eupdatedb.getOutput())

    # 4. check for security updates (if wanted)
    
    global blockList
    
    blockList = []
    secPackages = []
    
    if (mainConfig.getMainConfig('security')):
        checkSecurity = ShellExecution('glsa-check --list --nocolor')

        if (checkSecurity.getExitStatus() != 0):
            mail.earlyErrorMail('Problem during glsa-check --list',checkSecurity.getOutput())
        secPackages = sre.findall(" \[N\].*?\(\s(.*?)\s\)",checkSecurity.getOutput())
        for secPackage in secPackages:
            if (sre.search("\s",secPackage)):
                severalPackages = sre.split("\s",secPackage)
                while secPackage in secPackages:
                    secPackages.remove(secPackage)
                for singlePackage in severalPackages:
                    if (singlePackage != '...'):
                        secPackages.append(singlePackage)
    
        secPackages.reverse()
        if (int(mainConfig.getMainConfig('verbosity')) > 2): # debugging only
            print "glsa-package count:" , len(secPackages) 
            print "glsa-package list:" , secPackages

    packageList = sets.Set(secPackages)
    
    # Only prebuilt security packages if they are installed:
    newSecPList = packageList.copy()
    for package in newSecPList:
        securityCheckCommand = 'emerge --pretend ' + package
        securityCheck = ShellExecution(securityCheckCommand)
        if sre.findall("\[ebuild\s*N",securityCheck.getOutput()):
            packageList.discard(package)
    
    # 5. check for system/world
    
    if ((mainConfig.getMainConfig('update') == 'system') or (mainConfig.getMainConfig('update') == 'world')):
        os.putenv('NOCOLOR','true')
        emergePretendCommand = 'emerge --pretend ' + mainConfig.getMainConfig('sysworldoptions') + ' ' + mainConfig.getMainConfig('update')

        emergePretend = ShellExecution(emergePretendCommand)
        if (emergePretend.getExitStatus() != 0):
            mail.earlyErrorMail('Problem during ' + emergePretendCommand,emergePretend.getOutput())
        updateList = sre.findall("\[ebuild.*?\]\s(.*?)\s.*?\n",emergePretend.getOutput())
        blockList = sre.findall("\[blocks.*?\]\s(.*?)\s.*?\n",emergePretend.getOutput())
        updateList.reverse()
        for block in blockList:
            mail.addBlockPackage(block)
    elif (mainConfig.getMainConfig('update') == 'security'):
        pass

    else:
        mail.earlyErrorMail('unsupported value for update-option in config file','*** ERROR***\n\n\nglcu found an unsupported option in /etc/conf.d/glcu:\n\n  update: ' + mainConfig.getMainConfig('update') + '\n\nThis needs to be fixed for glcu to work!')
    

    # check for duplicates (with and without version numbers):
    cpPackList = packageList.copy()
    for package in cpPackList:
        for upackage in updateList:
            if sre.match(package,upackage):
                packageList.discard(package)
    
    # merge set from glsa-check and list from emerge system/world
    packageList.update(updateList)
    
    if (int(mainConfig.getMainConfig('verbosity')) > 2): # debugging only
        print 'Number of Packages:' , len(packageList)
        print 'Packages:' , packageList
        print 'Blocking Packages:' , blockList
        
    # 6. prebuilt all needed packages (security + system/world)
    
    # check for package directory
    pkgDir = '/usr/portage/packages'
    if (os.getenv('PKGDIR')):
        pkgDir = os.getenv('PKGDIR')    
    pkgDir = pkgDir + '/All'

    # get prebuilt packages in PKGDIR:
    prebuiltPackages = []
    prebuiltDirList = os.listdir(pkgDir)
    for dirPackage in prebuiltDirList:
        prebuiltPackage = dirPackage.replace('.tbz2','')
        prebuiltPackages.append(prebuiltPackage)

        

    # check for already existing prebuilt Packages
    newCpPList = packageList.copy()    
    for package in newCpPList:
        # remove package paths
        prePackage = sre.sub('.*/','',package)
        for prebuiltPackage in prebuiltPackages:
            if (sre.match(prePackage,prebuiltPackage)):
                if (int(mainConfig.getMainConfig('verbosity')) > 2): # debugging only
                    print "Removing package:" , package
                mail.addExistingPackage(package)
                packageList.discard(package)

        
    # exit if there are no packages to update
    if (len(packageList) == 0):
        if (int(mainConfig.getMainConfig('verbosity')) > 2): # debugging only
            print 'no packages to update. Exiting...'
        sys.exit(0)    

        
    # check for packages with dependencies (which can't be prebuild)
    depCpPList = packageList.copy()
    for depPackage in depCpPList:
        dependencyCheckCommand = 'emerge --pretend =' + depPackage
        checkDep = ShellExecution(dependencyCheckCommand)
        if (len(sre.findall("\[ebuild|\[blocks",checkDep.getOutput())) != 1):
            if (int(mainConfig.getMainConfig('verbosity')) > 2): # debugging only
                print "Removing package:" , depPackage
            mail.addPackage(depPackage,-1,checkDep.getOutput())
            packageList.discard(depPackage)
            

    # prebuilding packages        
    for package in packageList: 
        prebuildCommand = 'emerge --buildpkgonly --oneshot =' + package
        prebuild = ShellExecution(prebuildCommand)
        mail.addPackage(package,prebuild.getExitStatus(),prebuild.getOutput())


    
    # 7. save status to tmpfile for easy system update 
    #     and send eMail with report and HowTo update your system

    mail.writeMail()

    
    
    if (int(mainConfig.getMainConfig('verbosity')) > 2):
        print "\ncronExecution finished!\n\n"
    
def manualExecution ():
    # load configuration:
    global mainConfig
    mainConfig = GlcuConfig('manual')
    if (int(mainConfig.getMainConfig('verbosity')) > 2): # debugging only
        print "*** manualExecution: " , sys.argv[0]

    # check if glcu was started as root
    if (os.getenv('USER') != 'root'):
        exception('glcu can only be run as root!')
        
    ###########################    
    # easy update if update file is given
    
    if (len(sys.argv) == 2):
        updateFilename = sys.argv.pop()
        if (os.path.isfile(updateFilename) == 1):
    
            # 1. load update file
        
            updateFile = file(updateFilename,'r')
            updates = pickle.load(updateFile)
            updateFile.close()
    
            extra = None
            global noRemove
            noRemove = True
            # 2. Check for last element (security/system/world)
            if (updates[len(updates)-1] == 'security') or (updates[len(updates)-1] == 'system') or (updates[len(updates)-1] == 'world'):
                extra = updates.pop()
            
            rmUpdates = updates[:]
            
            print "\n\n*** Welcome to glcu's easy update feature ***\n\n"            
            
            
            # 3. install prebuilt packages
            if (mainConfig.getMainConfig('automatic') == False):
                    print "\n\nPrebuilt packages:\n------------------"
                    for printPackage in updates:
                        print printPackage
                    print "\n\nDo you want to install the prebuilt packages [Y/n]:"    
                    print "\n >" ,
                    extraAnswer = raw_input()
                    if (extraAnswer == 'Y') or (extraAnswer == 'y') or (extraAnswer==''):
                        updatePackages = ''
                        while (updates):
                            updatePackages = updatePackages + ' =' +updates.pop()
                        buildCommand = 'emerge --verbose --usepkgonly ' + updatePackages
                        buildExitStatus = os.system(buildCommand)
                        if (buildExitStatus == 0):
                            noRemove = False
            else:
                updatePackages = ''
                while (updates):
                    updatePackages = updatePackages + ' =' + updates.pop()
                buildCommand = 'emerge --verbose --usepkgonly ' + updatePackages
                buildExitStatus = os.system(buildCommand)
        
            # 4. ask for full install (if packages couldn't be prebuilt)
    
            if (extra):
                updateCommand = 'emerge --verbose ' + mainConfig.getMainConfig('sysworldoptions') + ' ' + extra
                showExitStatus = os.system(updateCommand + ' --pretend')
                if (mainConfig.getMainConfig('automatic') == False):
                    print "\n\nDo you want to install above packages now? [Y/n]\n >" ,
                    extraAnswer = raw_input()
                    if (extraAnswer == 'Y') or (extraAnswer == 'y') or (extraAnswer==''):
                        extraExitStatus = os.system(updateCommand)
                else:
                    extraExitStatus = os.system(updateCommand)
                    
            # 5. fix security
            if (mainConfig.getMainConfig('automatic') == False):
                print "\n\nDo you want to fix all glsa's now? [Y/n]\n >" ,
                extraAnswer = raw_input()
                if (extraAnswer == 'Y') or (extraAnswer == 'y') or (extraAnswer==''):
                    secCommand = 'glsa-check --fix new'
                    securityExitStatus = os.system(secCommand)
            else:
                secCommand = 'glsa-check --fix new'
                securityExitStatus = os.system(secCommand)
            
            # 6. remove updateFilename and binary packages if wanted:
            if (noRemove == False):
                os.remove(updateFilename)
                if mainConfig.getMainConfig('removeprebuilt'):
                    pkgDir = '/usr/portage/packages'
                    if (os.getenv('PKGDIR')):
                        pkgDir = os.getenv('PKGDIR')    
                    pkgDir = pkgDir + '/All/'
                    for packageName in rmUpdates:
                        packageShortName = sre.sub('.*/','',packageName)
                        packagePath = pkgDir + packageShortName + '*'
                        print "removing " + packagePath
                        os.system('rm ' + packagePath) 
            
        else:
            exception(updateFilename + ' is not a update file')

    
    ####################################    
    # manual installation if NO update file is given
    else:
        print "manual mode is not yet implemented"
        sys.exit(1)

    


    

if sre.match('/etc/cron',sys.argv[0]):
    cronExecution()
    
else:
    manualExecution()
    

